@@ -25,6 +25,7 @@
 #include <glib/gstdio.h>
 
 #include <sys/stat.h>
+#include <sys/sysmacros.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include <linux/loop.h>
@@ -569,7 +570,7 @@ udisks_state_check_mounted_fs_entry (UDisksState  *state,
   if (block_device_value == NULL)
     {
       s = g_variant_print (value, TRUE);
-      udisks_error ("mounted-fs entry %s is invalid: no block-device key/value pair", s);
+      udisks_critical ("mounted-fs entry %s is invalid: no block-device key/value pair", s);
       g_free (s);
       attempt_no_cleanup = FALSE;
       goto out;
@@ -580,7 +581,7 @@ udisks_state_check_mounted_fs_entry (UDisksState  *state,
   if (fstab_mount_value == NULL)
     {
       s = g_variant_print (value, TRUE);
-      udisks_error ("mounted-fs entry %s is invalid: no fstab-mount key/value pair", s);
+      udisks_critical ("mounted-fs entry %s is invalid: no fstab-mount key/value pair", s);
       g_free (s);
       attempt_no_cleanup = FALSE;
       goto out;
@@ -713,7 +714,7 @@ udisks_state_check_mounted_fs_entry (UDisksState  *state,
                                                       "umount -l %s",
                                                       escaped_mount_point))
             {
-              udisks_error ("Error cleaning up mount point %s: Error unmounting: %s",
+              udisks_critical ("Error cleaning up mount point %s: Error unmounting: %s",
                             mount_point, error_message);
               g_free (escaped_mount_point);
               g_free (error_message);
@@ -740,7 +741,7 @@ udisks_state_check_mounted_fs_entry (UDisksState  *state,
             {
               if (g_rmdir (mount_point) != 0)
                 {
-                  udisks_error ("Error cleaning up mount point %s: Error removing directory: %m",
+                  udisks_critical ("Error cleaning up mount point %s: Error removing directory: %m",
                                 mount_point);
                   /* keep the entry so we can clean it up later */
                   keep = TRUE;
@@ -786,7 +787,7 @@ udisks_state_check_mounted_fs (UDisksState *state,
     {
       udisks_warning ("Error getting mounted-fs: %s (%s, %d)",
                       error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -824,7 +825,7 @@ udisks_state_check_mounted_fs (UDisksState *state,
                           error->message,
                           g_quark_to_string (error->domain),
                           error->code);
-          g_error_free (error);
+          g_clear_error (&error);
           goto out;
         }
     }
@@ -879,7 +880,7 @@ udisks_state_add_mounted_fs (UDisksState    *state,
     {
       udisks_warning ("Error getting mounted-fs: %s (%s, %d)",
                       error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -898,7 +899,7 @@ udisks_state_add_mounted_fs (UDisksState    *state,
           /* Skip/remove stale entries */
           if (g_strcmp0 (entry_mount_point, mount_point) == 0)
             {
-              udisks_warning ("Removing stale entry for mount point `%s' in /run/udisks2/mounted-fs file",
+              udisks_warning ("Removing stale entry for mount point `%s' in /run/udisks/mounted-fs file",
                               entry_mount_point);
             }
           else
@@ -936,14 +937,14 @@ udisks_state_add_mounted_fs (UDisksState    *state,
   /* save new entries */
   error = NULL;
   if (!udisks_state_set (state,
-                           "mounted-fs",
-                           G_VARIANT_TYPE ("a{sa{sv}}"),
-                           new_value, /* consumes new_value */
-                           &error))
+                         "mounted-fs",
+                         G_VARIANT_TYPE ("a{sa{sv}}"),
+                         new_value, /* consumes new_value */
+                         &error))
     {
       udisks_warning ("Error setting mounted-fs: %s (%s, %d)",
                       error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -990,7 +991,7 @@ udisks_state_find_mounted_fs (UDisksState   *state,
     {
       udisks_warning ("Error getting mounted-fs: %s (%s, %d)",
                       error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -1105,7 +1106,7 @@ udisks_state_check_unlocked_luks_entry (UDisksState  *state,
   if (crypto_device_value == NULL)
     {
       s = g_variant_print (value, TRUE);
-      udisks_error ("unlocked-luks entry %s is invalid: no crypto-device key/value pair", s);
+      udisks_critical ("unlocked-luks entry %s is invalid: no crypto-device key/value pair", s);
       g_free (s);
       attempt_no_cleanup = TRUE;
       goto out;
@@ -1116,7 +1117,7 @@ udisks_state_check_unlocked_luks_entry (UDisksState  *state,
   if (dm_uuid_value == NULL)
     {
       s = g_variant_print (value, TRUE);
-      udisks_error ("unlocked-luks entry %s is invalid: no dm-uuid key/value pair", s);
+      udisks_critical ("unlocked-luks entry %s is invalid: no dm-uuid key/value pair", s);
       g_free (s);
       attempt_no_cleanup = TRUE;
       goto out;
@@ -1200,7 +1201,7 @@ udisks_state_check_unlocked_luks_entry (UDisksState  *state,
                                                       "cryptsetup luksClose %s",
                                                       escaped_device_file))
             {
-              udisks_error ("Error cleaning up LUKS device %s: %s",
+              udisks_critical ("Error cleaning up LUKS device %s: %s",
                             device_file_cleartext, error_message);
               g_free (escaped_device_file);
               g_free (error_message);
@@ -1253,7 +1254,7 @@ udisks_state_check_unlocked_luks (UDisksState *state,
     {
       udisks_warning ("Error getting unlocked-luks: %s (%s, %d)",
                       error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -1291,7 +1292,7 @@ udisks_state_check_unlocked_luks (UDisksState *state,
                           error->message,
                           g_quark_to_string (error->domain),
                           error->code);
-          g_error_free (error);
+          g_clear_error (&error);
           goto out;
         }
     }
@@ -1346,7 +1347,7 @@ udisks_state_add_unlocked_luks (UDisksState  *state,
     {
       udisks_warning ("Error getting unlocked-luks: %s (%s, %d)",
                       error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -1410,7 +1411,7 @@ udisks_state_add_unlocked_luks (UDisksState  *state,
     {
       udisks_warning ("Error setting unlocked-luks: %s (%s, %d)",
                       error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -1455,7 +1456,7 @@ udisks_state_find_unlocked_luks (UDisksState   *state,
     {
       udisks_warning ("Error getting unlocked-luks: %s (%s, %d)",
                       error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -1544,7 +1545,7 @@ udisks_state_check_loop_entry (UDisksState  *state,
     {
       gchar *s;
       s = g_variant_print (value, TRUE);
-      udisks_error ("loop entry %s is invalid: no backing-file key/value pair", s);
+      udisks_critical ("loop entry %s is invalid: no backing-file key/value pair", s);
       g_free (s);
       goto out;
     }
@@ -1632,7 +1633,7 @@ udisks_state_check_loop (UDisksState *state,
     {
       udisks_warning ("Error getting loop: %s (%s, %d)",
                       error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -1670,7 +1671,7 @@ udisks_state_check_loop (UDisksState *state,
                           error->message,
                           g_quark_to_string (error->domain),
                           error->code);
-          g_error_free (error);
+          g_clear_error (&error);
           goto out;
         }
     }
@@ -1726,7 +1727,7 @@ udisks_state_add_loop (UDisksState   *state,
     {
       udisks_warning ("Error getting loop: %s (%s, %d)",
                       error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -1789,7 +1790,7 @@ udisks_state_add_loop (UDisksState   *state,
     {
       udisks_warning ("Error setting loop: %s (%s, %d)",
                       error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -1834,7 +1835,7 @@ udisks_state_has_loop (UDisksState   *state,
     {
       udisks_warning ("Error getting loop: %s (%s, %d)",
                       error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -1978,7 +1979,7 @@ udisks_state_check_mdraid (UDisksState *state,
     {
       udisks_warning ("Error getting mdraid: %s (%s, %d)",
                       error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -2016,7 +2017,7 @@ udisks_state_check_mdraid (UDisksState *state,
                           error->message,
                           g_quark_to_string (error->domain),
                           error->code);
-          g_error_free (error);
+          g_clear_error (&error);
           goto out;
         }
     }
@@ -2064,7 +2065,7 @@ udisks_state_add_mdraid (UDisksState   *state,
     {
       udisks_warning ("Error getting mdraid: %s (%s, %d)",
                       error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -2083,7 +2084,7 @@ udisks_state_add_mdraid (UDisksState   *state,
           if (entry_raid_device == raid_device)
             {
               udisks_warning ("Removing stale entry for raid device %u:%u in /run/udisks2/mdraid file",
-                              major (raid_device), minor (raid_device));
+                                major (raid_device), minor (raid_device));
             }
           else
             {
@@ -2119,7 +2120,7 @@ udisks_state_add_mdraid (UDisksState   *state,
     {
       udisks_warning ("Error setting mdraid: %s (%s, %d)",
                       error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
+      g_clear_error (&error);
       goto out;
     }
 
